unit NewBoard;

interface

uses
	Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
	Dialogs, IdAntiFreezeBase, IdAntiFreeze, IdBaseComponent, IdComponent,
	IdTCPConnection, IdTCPClient, IdHTTP, IDException, StdCtrls, IniFiles,
	GikoSystem, BoardGroup, MojuUtils;

type
	TNewBoardItem = record
		FResponseCode: Integer;
		FContent: string;
	end;

	TNewBoardDialog = class(TForm)
		Label1: TLabel;
		MessageMemo: TMemo;
		UpdateButton: TButton;
	CloseButton: TButton;
		Indy: TIdHTTP;
		IdAntiFreeze: TIdAntiFreeze;
		StopButton: TButton;
		Label2: TLabel;
	BoardURLComboBox: TComboBox;
	Label13: TLabel;
	EditIgnoreListsButton: TButton;
		procedure UpdateButtonClick(Sender: TObject);
		procedure StopButtonClick(Sender: TObject);
		procedure CloseButtonClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
	procedure EditIgnoreListsButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
	private
		{ Private 錾 }
		IgnoreLists : TStringList;
		FAbort: Boolean;
		function BoardDownload: TNewBoardItem;
		procedure UpdateURL(s: string);
		procedure SetIgnoreCategory(b: boolean);
		procedure EditIgnoreList(Sender: TObject);
		procedure UpdateIgnoreList(Sender: TObject);
	public
		{ Public 錾 }
	end;

var
	NewBoardDialog: TNewBoardDialog;


implementation

uses Giko, IdHeaderList;

{$R *.dfm}

procedure TNewBoardDialog.UpdateButtonClick(Sender: TObject);
var
	Item: TNewBoardItem;
begin
	try
        GikoSys.Setting.BoardURLSelected := BoardURLComboBox.ItemIndex + 1;
		FAbort := False;
		UpdateButton.Enabled := False;
		StopButton.Enabled := True;
		CloseButton.Enabled := False;
		Item := BoardDownload;
		StopButton.Enabled := False;
		if FAbort then
			Exit;
		if Item.FContent <> '' then begin
			UpdateURL(Item.FContent);
			GikoForm.ReloadBBS;
		end else
			MessageMemo.Lines.Add('_E[hs܂[' + IntToStr(Item.FResponseCode) + ']');
	finally
		UpdateButton.Enabled := True;
		StopButton.Enabled := False;
		CloseButton.Enabled := True;
	end;
end;

procedure TNewBoardDialog.StopButtonClick(Sender: TObject);
begin
	FAbort := True;
	Indy.DisconnectSocket;
end;

procedure TNewBoardDialog.CloseButtonClick(Sender: TObject);
begin
	Close;
end;

function TNewBoardDialog.BoardDownload: TNewBoardItem;
var
	URL: string;
	Stream: TMemoryStream;
	s: string;
	i: Integer;
begin
	MessageMemo.Clear;
	Indy.Request.Clear;
	Indy.RecvBufferSize := Gikosys.Setting.RecvBufferSize;
	Indy.ProxyParams.BasicAuthentication := False;
	if GikoSys.Setting.ReadProxy then begin
		if GikoSys.Setting.ProxyProtocol then
			Indy.ProtocolVersion := pv1_1
		else
			Indy.ProtocolVersion := pv1_0;
		Indy.ProxyParams.ProxyServer := GikoSys.Setting.ReadProxyAddress;
		Indy.ProxyParams.ProxyPort := GikoSys.Setting.ReadProxyPort;
		Indy.ProxyParams.ProxyUsername := GikoSys.Setting.ReadProxyUserID;
		Indy.ProxyParams.ProxyPassword := GikoSys.Setting.ReadProxyPassword;
		if GikoSys.Setting.ReadProxyUserID <> '' then
			Indy.ProxyParams.BasicAuthentication := True;
	end else begin
		if GikoSys.Setting.Protocol then
			Indy.ProtocolVersion := pv1_1
		else
			Indy.ProtocolVersion := pv1_0;
		Indy.ProxyParams.ProxyServer := '';
		Indy.ProxyParams.ProxyPort := 80;
		Indy.ProxyParams.ProxyUsername := '';
		Indy.ProxyParams.ProxyPassword := '';
	end;
	//URL := GikoSys.Setting.BoardURL2ch;
    URL := BoardURLComboBox.Text;
	Indy.Request.UserAgent := GikoSys.GetUserAgent;
	Indy.Request.Referer := '';
	Indy.Request.AcceptEncoding := 'gzip';

	Indy.Request.CacheControl := 'no-cache';
	Indy.Request.CustomHeaders.Add('Pragma: no-cache');

//	s := '';
	Stream := TMemoryStream.Create;
	try
		try
			MessageMemo.Lines.Add('ȉ̏ꏊ擾܂');
			//MessageMemo.Lines.Add(GikoSys.Setting.BoardURL2ch);
            MessageMemo.Lines.Add(URL);
			MessageMemo.Lines.Add('_E[hJn܂');
			Indy.Get(URL, Stream);
			Result.FContent := GikoSys.GzipDecompress(Stream, Indy.Response.ContentEncoding);
			MessageMemo.Lines.Add('_E[h܂');
		except
			on E: EIdConnectException do begin
				MessageMemo.Lines.Add('');
				MessageMemo.Lines.Add('ڑs܂ vLVAFW̏Ԃ𒲂ׂĂ');
				MessageMemo.Lines.Add('FWĂl͐ݒmFĂ');
				MessageMemo.Lines.Add('NECPC̏ꍇPC GATEĂ\ł');
				MessageMemo.Lines.Add('Message: ' + E.Message);
			end;
			on E: Exception do begin
				if FAbort then
					MessageMemo.Lines.Add('_E[h𒆒f܂')
				else begin
					MessageMemo.Lines.Add('_E[hs܂');
					MessageMemo.Lines.Add('ResponseCode: ' + IntToStr(Indy.ResponseCode));
					MessageMemo.Lines.Add('Message: ' + E.Message);
					MessageMemo.Lines.Add('------------------------');
					for i := 0 to Indy.Response.RawHeaders.Count - 1 do begin
						s := Indy.Response.RawHeaders.Names[i];
						s := s + ': ' + Indy.Response.RawHeaders.Values[s];
						MessageMemo.Lines.Add(s);
					end;
					MessageMemo.Lines.Add('------------------------');
				end;
			end;
		end;
		Result.FResponseCode := Indy.ResponseCode;
	finally
		Stream.Free;
	end;
end;

procedure TNewBoardDialog.UpdateURL(s: string);
var
	i: Integer;
//	j: Integer;
	idx: Integer;
	idx1: Integer;
	idx2: Integer;
	tmp: string;
	URL: string;
	Title: string;
	cate: string;
	Board: TBoard;
	Change: Boolean;
	Ignore: Boolean;
	ini: TMemIniFile;
	oldURLs : TStringList;
	newURLs : TStringList;
begin
	Change := False;
	MessageMemo.Lines.Add('VAURLύX`FbNJn܂');
	MessageMemo.Lines.Add('');
	s := CustomStringReplace(s, '<B>', '<b>', true);
	s := CustomStringReplace(s, '<BR>', '<br>', true);
	s := CustomStringReplace(s, '</B>', '</b>', true);
	s := CustomStringReplace(s, '<A HREF', '<a href', true);
	s := CustomStringReplace(s, '</A', '</a', true);
	cate := '';

	oldURLs := TStringList.Create;
	newURLs := TStringList.Create;

	try

		GikoSys.ForceDirectoriesEx(GikoSys.GetConfigDir);
		ini := TMemIniFile.Create(GikoSys.GetBoardFileName);
		try
			//
			//폜IvVIĂꍇ̓NA

			ini.Clear;

			while True do begin
				idx1 := AnsiPos('<b>', s);
				idx2 := AnsiPos('<a', s);
				if (idx1 = 0) and (idx2 = 0) then Break;

				if idx1 < idx2 then begin
					//<br>
					idx := AnsiPos('</b>', s);
					if idx = 0 then begin
						s := Copy(s, idx1 + 4, Length(s));
						continue;
					end;
					tmp := Copy(s, idx1, (idx - idx1) + 4);
					tmp := CustomStringReplace(tmp, '<b>', '');
					tmp := CustomStringReplace(tmp, '</b>', '');
					Ignore := false;
					for i := 0 to IgnoreLists.Count - 1 do begin
						if tmp = Trim(IgnoreLists[i]) then begin
							cate := '';
							s := Copy(s, idx + 5, Length(s));
							Ignore := True;
							break;
						end;
					end;
					if Ignore then
						Continue;
					{
					if (tmp = '') or
						 (tmp = 'ʊ') or
						 (tmp = '܂aar') or
						 (tmp = '`bg') or
						 (tmp = 'G') or
						 (tmp = '^cē') or
						 (tmp = 'c[') or
						 (tmp = '̃TCg') then begin
						cate := '';
						s := Copy(s, idx + 5, Length(s));
						Continue;
					end;
					}
					s := Copy(s, idx + 5, Length(s));
					cate := tmp;
				end else begin
					//<a href=
					if cate = '' then begin
						s := Copy(s, idx2 + 2, Length(s));
					end else begin
						idx := AnsiPos('</a>', s);
						tmp := Copy(s, idx2, (idx - idx2) + 4);
						tmp := CustomStringReplace(tmp, '<a href=', '');
						tmp := CustomStringReplace(tmp, '</a>', '');
                        tmp := CustomStringReplace(tmp, 'TARGET=_blank', '');
						i := AnsiPos('>', tmp);
						if i <> 0 then begin
							URL := Copy(tmp, 1, i - 1);
							Title := Copy(tmp, i + 1, Length(tmp));
                            Board := BBSs[ 0 ].FindBoardFromTitle(Title);
                            if Board = nil then begin
                            	MessageMemo.Lines.Add('Vǉu' + Title + '(' + URL + ')v');
                                ini.WriteString(cate, Title, URL);
                                Change := True;
                            end else begin
                            	if Board.URL <> URL then begin
                                	MessageMemo.Lines.Add('URLύXu' + Board.Title + '(' + URL +')v');
                                    ini.WriteString(cate, Title, URL);
                                    oldURLs.Add(Board.URL);
                                    newURLs.Add(URL);
                                    Change := True;
                                end else begin
                                	ini.WriteString(cate, Title, URL);
                                end;
                            end;
						end else begin
							s := Copy(s, idx2 + 2, Length(s));
							Continue;
						end;
						s := Copy(s, idx + 5, Length(s));
					end;
				end;
			end;
		finally
			if Change then
				ini.UpdateFile;
			ini.Free;
		end;
		MessageMemo.Lines.Add('');
	    if Change then begin
            GikoForm.FavoritesURLReplace(oldURLs, newURLs);
            GikoForm.RoundListURLReplace(oldURLs, newURLs);
//            GikoForm.TabFileURLReplace(oldURLs, newURLs);
			MessageMemo.Lines.Add('VAURLύX`FbN܂');
			MessageMemo.Lines.Add('uv{^Ă');
		end else
			MessageMemo.Lines.Add('VAURLύX ܂ł');
    finally
    	oldURLs.Free;
    	newURLs.Free;
	end;
end;

procedure TNewBoardDialog.FormCreate(Sender: TObject);
begin
	StopButton.Enabled := False;
	BoardURLComboBox.Clear;
	BoardURLComboBox.Items.AddStrings(GikoSys.Setting.BoardURLs);
	try
		BoardURLComboBox.ItemIndex := GikoSys.Setting.BoardURLSelected - 1;
	except
		BoardURLComboBox.ItemIndex := 0;
	end;
	SetIgnoreCategory(false);
end;
//XV̏OJeSXg̓o^
{['', 'ʊ', '܂aar', '`bg', 'G', '^cē', 'c[', '̃TCg']}
procedure TNewBoardDialog.SetIgnoreCategory(b: boolean);
begin
	IgnoreLists := TStringList.Create;
	if not( FileExists(GikoSys.Setting.GetIgnoreFileName) ) or ( b )then begin
		IgnoreLists.Add('');
		IgnoreLists.Add('ʊ');
		IgnoreLists.Add('܂aar');
		IgnoreLists.Add('`bg');
		IgnoreLists.Add('G');
		IgnoreLists.Add('^cē');
		IgnoreLists.Add('c[');
		IgnoreLists.Add('̃TCg');
	end else begin
		try
			IgnoreLists.LoadFromFile(GikoSys.Setting.GetIgnoreFileName);
		except
			IgnoreLists.Free;
			SetIgnoreCategory(true);
		end;
	end;
end;

procedure TNewBoardDialog.EditIgnoreListsButtonClick(Sender: TObject);
begin
	EditIgnoreList(Sender);
	EditIgnoreListsButton.OnClick := UpdateIgnoreList;
end;
procedure TNewBoardDialog.EditIgnoreList(Sender: TObject);
var
	i: Integer;
begin
	EditIgnoreListsButton.Caption := 'OJeS[XV';
	Label2.Caption := 'ePsɃJeSLĂBisCtrl+Enterj';
	UpdateButton.Enabled := false;
	//MessageMemo.ReadOnly := false;
	MessageMemo.Clear;
	for i := 0 to IgnoreLists.Count - 1 do
		MessageMemo.Lines.Add(IgnoreLists[i]);
end;
procedure TNewBoardDialog.UpdateIgnoreList(Sender: TObject);
var
	i: Integer;
begin
	Label2.Caption := '݁AJĂ^uqXgȂǂ͔XVɃNA܂';
    UpdateButton.Enabled := true;
	EditIgnoreListsButton.Caption := 'OJeS[ҏW';
	IgnoreLists.Clear;
	for i := 0 to MessageMemo.Lines.Count - 1 do
		IgnoreLists.Add(MessageMemo.Lines[i]);
	IgnoreLists.SaveToFile(GikoSys.Setting.GetIgnoreFileName);
	IgnoreLists.Free;
	SetIgnoreCategory(false);
	//MessageMemo.ReadOnly := true;
	MessageMemo.Clear;
	EditIgnoreListsButton.OnClick := EditIgnoreListsButtonClick;
end;

procedure TNewBoardDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
	IgnoreLists.Free;
end;

end.
